#include "AEventSemaphore.hpp"
#include "ATrace.hpp"

// Public
// Constructors
AEventSemaphore::AEventSemaphore(bool signalEvent, // true means continu if waiting
                                 bool autoReset)   // true means reset manually
{
   // Development tracing
//   AFUNCTRACE_DEVELOP();

   name = "";
   createSignaled = signalEvent;
   createAutoReset = autoReset;
}

AEventSemaphore::AEventSemaphore(char* aName,
                                 AStartOption option,
                                 bool signalEvent,       // true means continu if waiting
                                 bool autoReset)         // true means reset manually
{
   // Development tracing
//   AFUNCTRACE_DEVELOP();

   name = aName;
   createSignaled = signalEvent;
   createAutoReset = autoReset;

   switch(option)
   {
      case noOps:
      {
         break;
      }
      case create:
      {
         createEvent();
         break;
      }
      case createAndWait:
      {
         createEvent();
         waitEvent();
         break;
      }
      case open:
      {
         openEvent();
         break;
      }
      case openAndSet:
      {
         openEvent();
         setEvent();
         break;
      }
      case openAndWait:
      {
         openEvent();
         waitEvent();
         break;
      }
      default:
      {
         break;
      }
   }
}

AEventSemaphore::AEventSemaphore(AStartOption option,
                                 bool signalEvent,       // true means continu if waiting
                                 bool autoReset)         // true means reset manually
{
   // Development tracing
//   AFUNCTRACE_DEVELOP();

   name = "";
   createSignaled = signalEvent;
   createAutoReset = autoReset;

   switch(option)
   {
      case noOps:
      {
         break;
      }
      case create:
      {
         createEvent();
         break;
      }
      case createAndWait:
      {
         createEvent();
         waitEvent();
         break;
      }
      case open:
      {
         openEvent();
         break;
      }
      case openAndSet:
      {
         openEvent();
         setEvent();
         break;
      }
      case openAndWait:
      {
         openEvent();
         waitEvent();
         break;
      }
      default:
      {
         break;
      }
   }
}

// Destructor
AEventSemaphore::~AEventSemaphore()
{
   // Development tracing
//   AFUNCTRACE_DEVELOP();
}

// Operators

// Accessors
const string& AEventSemaphore::getName() const
{
   return name;
}

const AHandle& AEventSemaphore::getHandle() const
{
   return handle;
}


// Mutators
void AEventSemaphore::setName(const char* aName)
{
   name = aName;
}

void AEventSemaphore::setName(const string& aName)
{
   name = aName;
}

void AEventSemaphore::setHandle( const AHandle aHandle)
{
   handle = aHandle;
}


// Other operations
void AEventSemaphore::createEvent()
{
   // Development tracing
//   AFUNCTRACE_DEVELOP();

   // This function created an event semaphore. If the initial state is set to true
   // any thread/proces that calls waitEvent() will block untill someone calls
   // setEvent().

   handle = CreateEvent(0,                   // Security pointer
                        createAutoReset,     // Manual reset
                        createSignaled,      // Initial state
                        name.length() > 0 ? name.c_str() : 0);       // Name of the event

   if(!handle.isValid())
   {
      unsigned long result = GetLastError();
      if(result == ERROR_ALREADY_EXISTS)
      {
         ALibBaseException exception("AEventSemaphore::createEvent: ERROR_ALREADY_EXISTS");
         ATRACE_DEVELOP(exception.text());
         ATHROW(exception);
      }else
      {
         string text = string("AEventSemaphore::createEvent, GetLastError = ") + integerToString((unsigned long)GetLastError());
         ALibBaseException exception(text.c_str());
         ATRACE_DEVELOP(exception.text());
         ATHROW(exception);
      }
   }
}

void AEventSemaphore::setEvent()
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   // The SetEvent function sets the state of the specified event object to signaled.

   // The state of a manual-reset event object remains signaled until it is
   // set explicitly to the nonsignaled state by the ResetEvent function.
   // Any number of waiting threads, or threads that subsequently begin wait operations for
   // the specified event object, can be released while the object's state is signaled.

  // The state of an auto-reset event object remains signaled until a
  // single waiting thread is released, at which time the system automatically sets the
  // state to nonsignaled. If no threads are waiting, the event object's state remains signaled.

   // After this any process or thread that has called waitEvent() will start or continue running.

   bool result = SetEvent(handle);
   if(result == false)
   {
      string text = string("SetEvent, GetLastError = ") + integerToString((unsigned long)GetLastError());
      ALibBaseException exception(text.c_str());
      ATRACE_DEVELOP(exception.text());
      ATHROW(exception);
   }
}


void AEventSemaphore::openEvent( char* aName,
                                 unsigned long accessFlag,
                                 bool inheritFlag)
{
   // Development tracing
//   AFUNCTRACE_DEVELOP();

   if( accessFlag != EVENT_ALL_ACCESS)
   {
      string text = string("accessFlag != EVENT_ALL_ACCESS");
      ALibBaseException exception(text.c_str());
      ATRACE_DEVELOP(exception.text());
      ATHROW(exception);
   }

//   EVENT_MODIFY_STATE
//   SYNCHRONIZE

   // The OpenEvent function enables multiple processes to open handles of the same event object.
   // The function succeeds only if some process has already created the event by using the
   // CreateEvent function. The calling process can use the returned handle in any function
   // that requires a handle of an event object, subject to the limitations of the access specified
   // in the fdwAccess parameter.

   // The handle can be duplicated by using the DuplicateHandle function. Use the
   // CloseHandle function to close the handle. The system closes the handle
   // automatically when the process terminates. The event object is destroyed when its last
   // handle has been closed

   string tempName = name;
   if(aName)
   {
      tempName = aName;
   }

   handle = OpenEvent(  accessFlag,
                        inheritFlag,
                        tempName.c_str());
   if(!handle.isValid())
   {
      string text = string("OpenEvent, GetLastError = ") + integerToString((unsigned long)GetLastError());
      ALibBaseException exception(text.c_str());
      ATRACE_DEVELOP(exception.text());
      ATHROW(exception);
   }
}

void AEventSemaphore::resetEvent()
{
   // Development tracing
//   AFUNCTRACE_DEVELOP();

   // The ResetEvent function sets the state of the specified event object to nonsignaled.

   // The state of a manual-reset event object remains signaled until it is
   // set explicitly to the nonsignaled state by the ResetEvent function.
   // Any number of waiting threads, or threads that subsequently begin wait operations for
   // the specified event object, can be released while the object's state is signaled.

   // The state of an auto-reset event object remains signaled until a
   // single waiting thread is released, at which time the system automatically sets the
   // state to nonsignaled. If no threads are waiting, the event object's state remains signaled.

   // After this any process or thread that calls waitEvent() will be blocked.

   bool result = ResetEvent(handle);
   if(result == false)
   {
      string text = string("ResetEvent, GetLastError = ") + integerToString((unsigned long)GetLastError());
      ALibBaseException exception(text.c_str());
      ATRACE_DEVELOP(exception.text());
      ATHROW(exception);
   }

}

void AEventSemaphore::waitEvent(unsigned long waitTime)
{
   // Development tracing
//   AFUNCTRACE_DEVELOP();

   // The WaitForSingleObject function returns when the specified object is
   // in the signaled state or when the time-out interval elapses.

   // The WaitForSingleObject function checks the current state of the specified object.
   // If the object's state is nonsignaled, the calling thread enters an efficient wait state.
   // The thread consumes very little processor time while waiting for one of the conditions
   // of the wait to be satisfied.

   // The system adjusts the state of some types of objects after the object's
   // signaled state satisfies a wait condition. For example, the count of a
   // semaphore object is decreased by one.

   unsigned long result = WaitForSingleObject( handle, waitTime );
   if (result == WAIT_FAILED)
   {
      ALibBaseException exception("WaitForSingleObject WAIT_FAILED");
      ATRACE_DEVELOP(exception.text());
      ATHROW(exception);
   }else
   {  // Success
      switch(result)
      {
         case WAIT_OBJECT_0:
         {
            // The state of the specified object is signaled.
            // This should be the normal case
            ATRACE_DEVELOP("WaitForSingleObject WAIT_OBJECT_0");
            break;
         }
         case WAIT_ABANDONED:
         {
            // The specified object is a mutex object that was not released by the thread that owned the
            // mutex object before the owning thread terminated. Ownership of the mutex object is granted
            // to the calling thread, and the mutex is set to nonsignaled.
            ATRACE_DEVELOP("WaitForSingleObject ABANDONED");
            break;
         }
         case WAIT_TIMEOUT:
         {
            // The time-out interval elapsed, and the object's state is nonsignaled.
            ALibBaseException exception("WaitForSingleObject WAIT_TIMEOUT");
            ATRACE_DEVELOP(exception.text());
            ATHROW(exception);
            break;
         }
         default:
         {
            ATRACE_DEVELOP("WaitForSingleObject UNKNOWN RESULT");
            break;
         }
      }
   }
      /*
      if( result == WAIT_ABANDONED)
      {
         // The specified object is a mutex object that was not released by the thread that owned the
         // mutex object before the owning thread terminated. Ownership of the mutex object is granted
         // to the calling thread, and the mutex is set to nonsignaled.
         ATRACE_DEVELOP("WaitForSingleObject ABANDONED");
      } else if (result == WAIT_TIMEOUT)
      {
         // The time-out interval elapsed, and the object's state is nonsignaled.
         ALibBaseException exception("WaitForSingleObject WAIT_TIMEOUT");
         ATRACE_DEVELOP(exception.text());
         ATHROW(exception);
      } else if (result != WAIT_OBJECT_0)
      {
         // The state of the specified object is signaled.
         ALibBaseException exception("WaitForSingleObject WAIT_OBJECT_0");
         ATRACE_DEVELOP(exception.text());
         ATHROW(exception);
      }
      */
}

// Protected
// Operations

// Private
// Operations
AEventSemaphore::AEventSemaphore(const AEventSemaphore& aAEventSemaphore)
{
   if (this!=&aAEventSemaphore)
   {
   }
}

AEventSemaphore& AEventSemaphore::operator=(const AEventSemaphore& aAEventSemaphore)
{
   if (this!=&aAEventSemaphore)
   {
   }
   return *this;
}


// End of class definition of AEventSemaphore
