#ifndef _AEventSemaphore_hpp_
#define _AEventSemaphore_hpp_

// Copyright (c) 1997-2000 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@askesis.nl

// Author : Joost Kraaijeveld
// Created: 14 Dec 1997 15:44:10
// Purpose:

// Configuration file
#include "ALibBase.h"

// Standard C/C++ library headers

// Operating system toolkit headers
#ifdef __ALIBWINDOWS__
#include <windows.h>
#endif // __ALIBWINDOWS__

// Other class headers
#include "AHandle.hpp"

// Forward declarations

class ALIBIMPEXP AEventSemaphore
{
   public:
      // Enum for start options
      typedef enum
      {
         noOps,
         create,
         createAndWait,
         open,
         openAndSet,
         openAndWait
      }AStartOption;

      // Constructors
      AEventSemaphore(  bool signalEvent = false,  // true means continue if waiting
                        bool autoReset = true);    // true means reset manually
      // Typical first call. With default params it will create an eventsemaphore that will block
      // on a call to waitEvent
      AEventSemaphore(  char* aName,
                        AStartOption option,
                        bool signalEvent = false,     // true means continue if waiting
                        bool autoReset = true);       // true means reset manually
      AEventSemaphore(  AStartOption option,
                        bool signalEvent = false,     // true means continue if waiting
                        bool autoReset = true);       // true means reset manually


      // Destructor
      virtual ~AEventSemaphore();

      // Operators

      // Accessors
      const string&                             getName() const;
      const AHandle&                            getHandle() const;

      // Mutators
      void                                      setName( const char* aName);
      void                                      setName( const string& aName);
      void                                      setHandle(  const AHandle aHandle);

      // Other operations
      void                                      createEvent();
      void                                      setEvent();
      void                                      openEvent(  char* aName = 0,
                                                            unsigned long accessFlag = EVENT_ALL_ACCESS,
                                                            bool inheritFlag = false);

      void                                      resetEvent();
      void                                      waitEvent(unsigned long waitTime = INFINITE);

      // Attributes

   protected:
      // Attributes

      // Operations

   private:
      // Friends

      // Attributes
      string                                    name;
      AHandle                                   handle;
      bool                                      createSignaled;
      bool                                      createAutoReset;

      // Operations
      AEventSemaphore(const AEventSemaphore& aAEventSemaphore);
      AEventSemaphore& operator=(const AEventSemaphore& aAEventSemaphore);

}; // End of class declaration of AEventSemaphore
#endif // _AEventSemaphore_hpp_
