#include "AException.hpp"
#include "APlatform.hpp"

// Copyright (c) 1997-2000 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@Askesis.nl

// Author : Joost Kraaijeveld
// Created: 26-09-97 14:12:24

///////////////////////////////////////
class AExceptStaticPtr
{
   public:
      AExceptStaticPtr();
      ~AExceptStaticPtr();

      AException::TraceFn&                      traceFunction();

      AException::TraceFn*                      pTraceFn;
      AException::TraceFn*                      pUserTraceFn;

}; // IExceptStaticPtr

AExceptStaticPtr::AExceptStaticPtr()
   :  pTraceFn(0),
      pUserTraceFn(0)
{
}

AExceptStaticPtr::~AExceptStaticPtr()
{
}

///////////////////////////////////////
class AExceptTraceFn : public AException::TraceFn
{
}; // IExceptTraceFn


///////////////////////////////////////
AExceptTraceFn defaultTraceFn;
static AExceptStaticPtr AExceptStatics;

///////////////////////////////////////
class AExcText
{
   public:
      AExcText(const char* errText,
               const AExcText* msgtxtOld );
      AExcText(const AExcText& msg );

      ~AExcText()
      {
         if (pszClMsg) delete [] pszClMsg;
         if (msgtxtClNext) delete msgtxtClNext;
      }

      const char*                               text() const
      {
         return pszClMsg;
      }

      AExcText*                                 next() const
      {
         return msgtxtClNext;
      }

      void                                      appendText( const char* errText);

      char*                                     pszClMsg;
      AExcText*                                 msgtxtClNext;

}; // AExcText

AException::TraceFn& AExceptStaticPtr::traceFunction()
{
  if(pUserTraceFn) return *pUserTraceFn;
  if(!pTraceFn)
  {
     pTraceFn = &defaultTraceFn;
  }
  return *pTraceFn;
}

AExcText::AExcText(  const char* errText,
                     const AExcText* msgtxtOld )
{
   void(*poldnh)() = set_new_handler(0);
   pszClMsg = new char[strlen(errText)+1];
   if (pszClMsg) strcpy(pszClMsg, errText);
   msgtxtClNext = (AExcText*)msgtxtOld;
   set_new_handler(poldnh);
}

AExcText::AExcText ( const AExcText& msg )
{
   void(*poldnh)() = set_new_handler(0);
   pszClMsg = new char[strlen(msg.pszClMsg)+1];
   if (pszClMsg)
   {
      strcpy(pszClMsg, msg.pszClMsg);
      if (msg.msgtxtClNext)
      {
         msgtxtClNext = new AExcText(*(msg.msgtxtClNext));
      }else
      {
         msgtxtClNext = 0;
      }
    }else
    {
       pszClMsg = msg.pszClMsg;
       msgtxtClNext = msg.msgtxtClNext;
    }
    set_new_handler(poldnh);
}

void AExcText::appendText ( const char* errText )
{
   void(*poldnh)() = set_new_handler(0);
   char* pszText;
   if (pszClMsg)
   {
      pszText = new char[strlen(pszClMsg)+strlen(errText)+1];
   } else
   {
      pszText = new char[strlen(errText)+1];
   }

   if (pszText)
   {
      if (pszClMsg)
      {
         strcpy(pszText, pszClMsg);
         delete [] pszClMsg;
         strcat(pszText, errText);
         pszClMsg = pszText;
      }else
      {
         strcpy(pszText, errText);
         pszClMsg = pszText;
      }
   }
   set_new_handler(poldnh);
}

/*static*/ AException::ErrorCodeGroup const AException::baseLibrary          = "IBM Class Library";
/*static*/ AException::ErrorCodeGroup const AException::CLibrary             = "Standard C Library";
/*static*/ AException::ErrorCodeGroup const AException::operatingSystem      = "Operating System";
/*static*/ AException::ErrorCodeGroup const AException::presentationSystem   = "Presentation System";
/*static*/ AException::ErrorCodeGroup const AException::other                = "Other Error Group";


static const char* headerText = "%s exception thrown.\n   function: %s\n   file: %s\n   line: %d";

// public
// Constructors
AExceptionLocation::AExceptionLocation(const char* aFileName,
                                       const char* aFunctionName,
                                       unsigned long aLineNumber)
   :  file(aFileName),
      function(aFunctionName),
      line(aLineNumber)
{
}

// Destructor

// Operators

// Accessors
const char* AExceptionLocation::fileName ( ) const
{
  return file;
}

const char* AExceptionLocation::functionName ( ) const
{
   if (function) return function;
   return "Unknown";
}

unsigned long AExceptionLocation::lineNumber ( ) const
{
  return line;
}

// Mutators

// Operations

// protected
// Operations

// private

// Operations

// End of class definition of AExceptionLocation

// public
// Constructors
AException::AException( const char* anErrorText,
                        unsigned long anErrorId,
                        Severity aSeverity)
   :  exceptionSeverity(aSeverity),
      exceptionErrorId(anErrorId),
      exceptionLocationCount(0),
      exceptionErrorCodeGroup(baseLibrary)
{
   if (anErrorText != 0)
   {
      // Set the new handler to 0 to prevent extra exceptions
      void(*oldNewHandler)() = set_new_handler(0);

      exceptionText = new AExcText(anErrorText, 0);
      if (exceptionText)
      {
         exceptionTextLevelCount = 1;
      }else
      {
         exceptionTextLevelCount = 0;
      }

      // Restore the original new handler
      set_new_handler(oldNewHandler);
   }else         // null pointer to msg passed
   {
      exceptionTextLevelCount = 0;
      exceptionText = 0;
   }
}

AException::AException(const AException& aAException)
{
   exceptionSeverity       = aAException.exceptionSeverity;
   exceptionErrorId        = aAException.exceptionErrorId;
   exceptionLocationCount  = aAException.exceptionLocationCount;
   exceptionTextLevelCount = aAException.exceptionTextLevelCount;
   exceptionErrorCodeGroup = aAException.exceptionErrorCodeGroup;

   if (aAException.exceptionText != 0)
   {
      exceptionText = new AExcText(*(aAException.exceptionText));
   } else
   {
      exceptionText = 0;
   }

   for (unsigned long i=0; i<aAException.locationCount(); i++ )
   {
      exceptionLocation[i] = aAException.exceptionLocation[i];
   }
}

// Destructor
AException::~AException()
{
   if (exceptionText) delete exceptionText;
}

// Operators

// Accessors
AException::ErrorCodeGroup AException::errorCodeGroup ( ) const
{
   return exceptionErrorCodeGroup;
}

const char* AException::text(unsigned long anIndexFromTop) const
{
   AExcText* tmp = exceptionText;
   if (textCount() > anIndexFromTop)
   {
      for (unsigned long i = 0; i < anIndexFromTop; ++i)
      {
         tmp = tmp->next();
      } /* endfor */
      return tmp->text();
   } else
   {
      return 0;
   }
}

unsigned long  AException::textCount() const
{
   return exceptionTextLevelCount;
}

unsigned long AException::errorId() const
{
  return exceptionErrorId;
}

// Mutators
AException& AException::setErrorCodeGroup ( ErrorCodeGroup anErrorGroup )
{
   exceptionErrorCodeGroup = anErrorGroup;
   return *this;
}

// Operations
AException& AException::appendText(const char* anErrorText)
{
   if (anErrorText != 0)
   {
      if (exceptionText)
      {
        exceptionText->appendText(anErrorText);
      }else
      {
         setText(anErrorText);
      }
   }
   return *this;
}

AException& AException::setText(const char* anErrorText)
{
   if (anErrorText != 0)
   {
      // Set the new handler to 0 to prevent extra exceptions
      void(*oldNewHandler)() = set_new_handler(0);

      AExcText* msgtxtOld = exceptionText;
      exceptionText = new AExcText(anErrorText, msgtxtOld);
      if (exceptionText)
      {
         exceptionTextLevelCount++;
      }else
      {
         exceptionText= msgtxtOld;
      }

      // Restore the original new handler
      set_new_handler(oldNewHandler);
   }
   return *this;
}

AException& AException::setSeverity(Severity aSeverity)
{
   exceptionSeverity = aSeverity;
   return *this;
}

AException& AException::setErrorId(unsigned long anErrorId)
{
   exceptionErrorId = anErrorId;
   return *this;
}

AException& AException::addLocation(const AExceptionLocation& aAExceptionLocation)
{
   if(exceptionLocationCount == 5) exceptionLocationCount--;          // reset to fill in last entry
   exceptionLocation[exceptionLocationCount] = aAExceptionLocation;
   exceptionLocationCount++;
   return *this;
}

int AException::isRecoverable() const
{
   if(exceptionSeverity) return 1;
   return 0;
}

const char* AException::name() const
{
   return "AException";
}

unsigned long AException::locationCount() const
{
   return exceptionLocationCount;
}

const AExceptionLocation* AException::locationAtIndex(unsigned long aLocationIndex) const
{
   if (aLocationIndex < exceptionLocationCount)
   {
     return &(exceptionLocation[aLocationIndex]);
   }
   return 0;
}

AException& AException::logExceptionData ( )
{
   // ??????
   AExceptStatics.traceFunction().logData(*this);
   return *this;
}

AException::TraceFn* AException::setTraceFunction( AException::TraceFn& aTraceFunction)
{
   AException::TraceFn* oldFunction = &(AExceptStatics.traceFunction());
   AExceptStatics.pUserTraceFn = &aTraceFunction;
   return oldFunction;
}

void AException::terminate ( )
{
  ::abort();
}

void AException::assertParameter(const char* anExceptionText,
                                 AExceptionLocation aAExceptionLocation)
{
   AAssertionFailure invParm(anExceptionText);
   invParm.addLocation(aAExceptionLocation);
   invParm.setErrorCodeGroup(other);
   invParm.logExceptionData();
   throw invParm;
}
// End of class definition of AException

// public
// Constructors

// Destructor

// Operators

// Accessors

// Mutators

// Operations
void AException::TraceFn::write ( const char* aBuffer )
{
   fprintf(stderr,"%s\n", aBuffer);
}

void AException::TraceFn::logData ( AException& aAException )
{
   // Set the new handler to 0 to prevent extra exceptions
   void(*oldNewHandler)() = set_new_handler(0);

   // Determine the largest buffer we will need.
   unsigned long maxLength = 200;
   unsigned long length = 0;
   if (aAException.exceptionTextLevelCount != 0)
   {
      length = strlen(headerText) + 1 +
               strlen(aAException.name()) +
               strlen(aAException.exceptionLocation[aAException.exceptionLocationCount - 1].functionName()) +
               strlen(aAException.exceptionLocation[aAException.exceptionLocationCount - 1].fileName()) + 8;
   }

   if (length > maxLength) maxLength = length;

   if (aAException.exceptionText)
   {
      for (int i = (aAException.textCount()-1); i >= 0 ;i-- )
      {
         length = strlen(aAException.text(i)) + 6;
         if (length > maxLength) maxLength = length;
      } /* endfor */
   }

   char * pbuf = new char[maxLength + 1];

   if(pbuf)
   {
      if (aAException.exceptionTextLevelCount != 0)
      {
         sprintf( pbuf,
                  headerText,
                  aAException.name(),
                  aAException.exceptionLocation[aAException.exceptionLocationCount - 1].functionName(),
                  aAException.exceptionLocation[aAException.exceptionLocationCount - 1].fileName(),
                  aAException.exceptionLocation[aAException.exceptionLocationCount - 1].lineNumber());
      }else
      {
         sprintf(pbuf,"\n%s exception thrown.\n   Location information is unavailable.", aAException.name());
      }
      AExceptStatics.traceFunction().write(pbuf);

      if (aAException.errorId())
      {
         sprintf(pbuf,"   Error Id is %lu", aAException.errorId());
      }else
      {
         sprintf(pbuf,"   Error Id is unavailable.");
      }
      AExceptStatics.traceFunction().write(pbuf);
      if (aAException.errorCodeGroup())
      {
         sprintf(pbuf,"   Error Code group is %s", aAException.errorCodeGroup());
      }else
      {
         sprintf(pbuf,"    Error Code group is unavailable.");
      }
      AExceptStatics.traceFunction().write(pbuf);
      if (aAException.exceptionText)
      {
         sprintf(pbuf,"   Exception text is:");
         AExceptStatics.traceFunction().write(pbuf);
         for (int i = (aAException.textCount()-1); i >= 0 ;i-- )
         {
            sprintf(pbuf,"      %s", aAException.text(i));
            AExceptStatics.traceFunction().write(pbuf);
         } /* endfor */
      }else
      {
         sprintf(pbuf,"   Exception text is unavailable.");
         AExceptStatics.traceFunction().write(pbuf);
      }
      delete [] pbuf;
      exceptionLogged();
   }else                       // new failed, no buffer available
   {
      if (!APlatform::isWinNT())
      {
         if (aAException.exceptionLocationCount != 0)
         {
            fprintf( stdout,
                     "%s exception thrown.\n   function: %s\n   file: %s\n   line: %lu.\n",
                     aAException.name(),
                     aAException.exceptionLocation[aAException.exceptionLocationCount - 1].functionName(),
                     aAException.exceptionLocation[aAException.exceptionLocationCount - 1].fileName(),
                     aAException.exceptionLocation[aAException.exceptionLocationCount - 1].lineNumber());
         }else
         {
            fprintf( stdout,
                     "\n%s exception thrown.\n   Location information is unavailable.\n",
                     aAException.name());

            if (aAException.errorId())
            {
               fprintf(stdout,"   Error ID is %lu\n", aAException.errorId());
            } else
            {
               fprintf(stdout,"   Error ID is unavailable.\n");
            }

            if (aAException.errorCodeGroup())
            {
               fprintf(stdout, " Error Code group is %s\n", aAException.errorCodeGroup());
            }else
            {
               fprintf(stdout,"   Error Code group is unavailable.\n");
            }

            if (aAException.exceptionText)
            {
               fprintf(stdout,"   Exception text is:\n");
               for (int i = (aAException.textCount()-1); i >= 0 ;i-- )
               {
                  fprintf(stdout,"      %s\n", aAException.text(i));
               } /* endfor */
            }else
            {
               fprintf(stdout,"   Exception text is unavailable.\n");
            }
         }

         if (aAException.exceptionTextLevelCount != 0)
         {
            fprintf( stderr,
                     "%s exception thrown.\n   function: %s\n   file: %s\n   line: %lu.\n",
                     aAException.name(),
                     aAException.exceptionLocation[aAException.exceptionTextLevelCount - 1].functionName(),
                     aAException.exceptionLocation[aAException.exceptionTextLevelCount - 1].fileName(),
                     aAException.exceptionLocation[aAException.exceptionTextLevelCount - 1].lineNumber());
         }else
         {
            fprintf( stderr,
                     "\n%s exception thrown.\n   Location information is unavailable.\n",
                     aAException.name());
         }

         if (aAException.errorId())
         {
            fprintf(stderr,"   Error ID is %lu\n", aAException.errorId());
         }else
         {
            fprintf(stderr,"   Error ID is unavailable.\n");
         }

         if (aAException.errorCodeGroup())
         {
            fprintf(stderr," Error Code group is %s\n", aAException.errorCodeGroup());
         }else
         {
            fprintf(stderr,"   Error Code group is unavailable.\n");
         }

         if(aAException.exceptionText)
         {
            fprintf(stderr,"   Exception text is:\n");
            for (int i = (aAException.textCount()-1); i >= 0 ;i-- )
            {
               fprintf(stderr,"      %s\n", aAException.text(i));
            } /* endfor */
         }else
         {
            fprintf(stderr,"   Exception text is unavailable.\n");
         }
      } /* endif */
   }
   // Restore the original new handler
   set_new_handler(oldNewHandler);
}
// protected
// Operations
AException::TraceFn::TraceFn ( )
{ }

void AException::TraceFn::exceptionLogged ( )
{ }

// private
// Operations

// End of class definition of AException::TraceFn


ALIBEXCLASSIMPLEMENT(AAssertionFailure,AException,"ALibIBM");
