#ifndef _AException_hpp_
#define _AException_hpp_

// Copyright (c) 1997-2000 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@Askesis.nl

// Author : Joost Kraaijeveld
// Created: 26-09-97 14:12:24
// Purpose: IBM compatible exception stuff. (I am not sure whether this is legal as it a rather shameless copy.)
//          Keep the interface identical so that source can be mixed (e.g. macros).

// Configuration file
#include "ALibConfig.h"

// Standard C/C++ library headers
#include <new>
#include <stdio.h>
#include <string.h>

// Operating system toolkit headers

// STL headers

// Other class headers

// Forward declarations
class AExcText;

class ALIBIMPEXP AExceptionLocation
{
   public:
      // Constructors
      AExceptionLocation ( const char* aFileName = 0,
                           const char* aFunctionName = 0,
                           unsigned long aLineNumber = 0 );

      // Destructor

      // Operators

      // Accessors
      const char*                               fileName() const;
      const char*                               functionName() const;
      unsigned long                             lineNumber() const;

      // Mutators

      // Operations

      // Attributes

   protected:
      // Attributes

      // Operations

   private:
      // Friends

      // Operations

      // Attributes
      const char*                               file;
      const char*                               function;
      unsigned long                             line;
}; // End of class declaration of AExceptionLocation

class ALIBIMPEXP AException
{
   public:
      enum Severity { unrecoverable, recoverable };
      typedef const char* ErrorCodeGroup;

      class ALIBIMPEXP TraceFn
      {
         public:
            // Constructors

            // Destructor

            // Operators

            // Accessors

            // Mutators

            // Operations
            virtual void                        write(const char* aBuffer);
            virtual void                        logData( AException& aAException);

            // Attributes

         protected:
            // Attributes

            // Operations
            TraceFn();
            virtual void                        exceptionLogged();

         private:
            // Friends

            // Operations

            // Attributes
            const char*                         file;
            const char*                         function;
            unsigned long                       line;
      }; // End of class declaration of TraceFn

      // Constructors
      AException (const char*       anErrorText,
                  unsigned long     anErrorId = 0,
                  Severity          aSeverity = AException::unrecoverable );
      AException( const AException& aAException );

      // Destructor
      virtual ~AException();

      // Operators

      // Accessors
      ErrorCodeGroup                            errorCodeGroup() const;
      const char*                               text( unsigned long anIndexFromTop = 0 ) const;
      unsigned long                             textCount() const;
      unsigned long                             errorId() const;
      const AExceptionLocation*                 locationAtIndex(  unsigned long aLocationIndex) const;

      // Mutators
      AException&                               setErrorCodeGroup(ErrorCodeGroup anErrorGroup);
      AException&                               appendText( const char* anErrorText);
      AException&                               setText( const char* anErrorText);
      AException&                               setSeverity(Severity aSeverity);
      AException&                               setErrorId( unsigned long anErrorId);
      virtual AException&                       addLocation( const AExceptionLocation& aAExceptionLocation);
      static AException::TraceFn*               setTraceFunction( AException::TraceFn& aTraceFunction);

      // Operations
      virtual const char*                       name() const;
      virtual int                               isRecoverable() const;
      unsigned long                             locationCount() const;
      virtual AException&                       logExceptionData();
      virtual void                              terminate();
      static void                               assertParameter(  const char* anExceptionText,
                                                                  AExceptionLocation aAExceptionLocation);

      // Attributes
      static ErrorCodeGroup const ALIBIMPEXP baseLibrary;
      static ErrorCodeGroup const ALIBIMPEXP CLibrary;
      static ErrorCodeGroup const ALIBIMPEXP operatingSystem;
      static ErrorCodeGroup const ALIBIMPEXP presentationSystem;
      static ErrorCodeGroup const ALIBIMPEXP other;

   protected:
      // Attributes

      // Operations

   private:
      // Friends
      friend class AException::TraceFn;

      // Operations
      AException& operator=(const AException& exc);

      // Attributes
      Severity                                  exceptionSeverity;         // Severity             exsevCl
      unsigned long                             exceptionErrorId;          // unsigned long        ulClErrorId
      AExceptionLocation                        exceptionLocation[5];      // IExceptionLocation   exlocClArray[5]
      unsigned long                             exceptionLocationCount;    // unsigned long        ulexlocClCount
      AExcText*                                 exceptionText;             // IExcText*            msgtxtClTop
      unsigned long                             exceptionTextLevelCount;   // unsigned long        ulClTxtLvlCount
      ErrorCodeGroup                            exceptionErrorCodeGroup;   // ErrorCodeGroup       fErrorGroup

   private:
}; // End of class declaration of AException

#ifdef __IBMCPP__
#define AEXCEPTION_LOCATION()   AExceptionLocation(__FILE__, __FUNCTION__, __LINE__)
#endif
#if defined(__CYGWIN32__) || defined(__MINGW32__)
#define AEXCEPTION_LOCATION()   AExceptionLocation(__FILE__, __PRETTY_FUNCTION__, __LINE__)
#endif

#define ALIBEXCLASSDECLARE(child,parent)                         \
class ALIBIMPEXP child : public parent                           \
{                                                                \
   public:                                                       \
      child(const char* a,                                       \
            unsigned long b = 0,                                 \
            Severity c = AException::unrecoverable);             \
      child(const child &);                                      \
      virtual ~child();                                          \
      virtual const char* name() const;                          \
   private:                                                      \
      child &operator = ( const child & );                       \
}

#define ALIBEXCLASSIMPLEMENT(child,parent,library)               \
child::child(  const char* a,                                    \
               unsigned long b,                                  \
               Severity c)                                       \
   :  parent(a, b, c)                                            \
{                                                                \
   setErrorCodeGroup( # library);                                \
}                                                                \
const char* child::name() const                                  \
{                                                                \
   return ( # child);                                            \
}                                                                \
child::~child()                                                  \
{                                                                \
}                                                                \
child::child(const child & a)                                    \
   :  parent(a)                                                  \
{                                                                \
   setErrorCodeGroup( # library);                                \
}

ALIBEXCLASSDECLARE(AAssertionFailure,AException);

#if defined(ADEVELOP)
  #define ASSERT(test)     if(!(test))                                                                                                 \
                           {                                                                                                           \
                              AException::assertParameter(  "The following expression must be true, but evaluated to false: " #test,   \
                                                            AEXCEPTION_LOCATION());                                                    \
                           }
#else
  #define ASSERT(test)
#endif

#define ATHROW(exc)        exc.addLocation(AEXCEPTION_LOCATION()),    \
                           exc.logExceptionData(),                    \
                           throw(exc)

#define ARETHROW(exc)      exc.addLocation(AEXCEPTION_LOCATION()),    \
                           exc.logExceptionData(),                    \
                           throw


#endif //_AException_hpp_
