#include "ALibString.hpp"

void stripLeadingSpaces(string& aString)
{
   // No spaces in a zero length string...
   if(aString.length() == 0) return;

   const char* startOfRawString  = aString.data();
   const char* endOfRawString    = startOfRawString + aString.length();

   unsigned long startPos        = 0;

   // Strip the leading spaces of string
   while(isspace(*(startOfRawString+startPos)))
   {
      ++startPos;
   }

   // Build the string
   const char* startOfStrippedString   = startOfRawString + startPos;
   const char* endOfStrippedString     = endOfRawString;
   aString = string(startOfStrippedString,(endOfStrippedString - startOfStrippedString));
}

void stripTrainingSpaces(string& aString)
{
   // No spaces in a zero length string...
   if(aString.length() == 0) return;

   const char* startOfRawString  = aString.data();
   const char* endOfRawString    = startOfRawString + aString.length();

   unsigned long endPos          = 0;

   // Strip the trailing spaces of string
   while(isspace(*(endOfRawString-endPos-1)))
   {
      ++endPos;
   }

   // Build the string
   const char* startOfStrippedString   = startOfRawString;
   const char* endOfStrippedString     = endOfRawString - endPos;
   aString = string(startOfRawString,(endOfStrippedString - startOfStrippedString));
}

void stringLeadingAndTrailingSpaces(string& aString)
{
   stripLeadingSpaces(aString);
   stripTrainingSpaces(aString);
}

string integerToString( unsigned long anInt)
{
   // max unsigned == 4294967295, ergo 10 chars
   char intAsChar[11];// = {0};
//   sprintf(&intAsChar[0],"%lu",anInt);
   sprintf(intAsChar,"%lu",anInt);
   return string(intAsChar);
}

string integerToString( long anInt)
{
   // max unsigned == 4294967295, ergo 10 chars
   char intAsChar[11];// = {0};

   //   sprintf(&intAsChar[0],"%ld",anInt);
   sprintf(intAsChar,"%ld",anInt);
   return string(intAsChar);
}

unsigned long stringToUnsignedLong(string& aString)
{
   // atol return 0L if the string cannot be converted to a number.
   // So check whether all chars in the string are digits. If not, throw an exception.
   for(unsigned long i = 0;i<aString.length();i++)
   {
      // isdigit returns 0 if the char is no digit
      if( isdigit(aString[i]) == 0)
      {
         // char string may contain a sign!!!
         // This conversion only allows it to be the very first char.
         // No blanks are allowed before the sign.
         if( ((aString[i] == '-') || (aString[i] == '+')) && i == 0) break;

         string excText = string("Cannot convert \"") + aString + string("\" to a unsigned long.");
         ALibStringException exc(excText.c_str());
         ATHROW(exc);
      }
   }
   return atol(aString.c_str());
}

long stringToLong(string& aString)
{
   // atol return 0L if the string cannot be converted to a number.
   // So check whether all chars in the string are digits. If not, throw an exception.
   for(unsigned long i = 0;i<aString.length();i++)
   {
      // isdigit returns 0 if the char is no digit
      if( isdigit(aString[i]) == 0)
      {
         // char string may contain a sign!!!
         // This conversion only allows it to be the very first char.
         // No blanks are allowed before the sign.
         if( ((aString[i] == '-') || (aString[i] == '+')) && i == 0) break;

         string excText = string("Cannot convert \"") + aString + string("\" to a long.");
         ALibStringException exc(excText.c_str());
         ATHROW(exc);
      }
   }
   return atol(aString.c_str());
}

// These functions should be replaced by a parser class some day as they are tied to the ATextProfile class
void splitString( const string& aStringToSplit, string& aFirstString, const char aToken, string& aSecondString)
{
   const char* stringToSplit     = aStringToSplit.data();
   unsigned long stringLength    = aStringToSplit.length();
   unsigned long currentPosition = 0;

   // Find the first occurence of aToken
   while((*(stringToSplit+currentPosition) != aToken))
   {
      if(currentPosition == stringLength) // Token not found
      {
         aFirstString = aStringToSplit;
         aSecondString = "";
         return;
      }
      currentPosition++;
   }
   aFirstString   = string(stringToSplit,currentPosition);

   // Eat the token
   currentPosition++;

   aSecondString  = string(stringToSplit+currentPosition, stringLength-currentPosition);
}

void splitString( const string& aStringToSplit, string& aFirstString, const char aFirstToken, string& aSecondString, const char aSecondToken, string& aThirdString)
{
   string remainder;
   splitString(aStringToSplit, aFirstString, aFirstToken, remainder);
   splitString(remainder, aSecondString, aSecondToken,aThirdString);
}

string bracketize(const string& aString)
{
   string bracketizedString(aString);

   if (bracketizedString[0] != '[')
   {
      bracketizedString = string((char*)"[") + bracketizedString + string((char*)"]");
   }
   return bracketizedString;
}

string bracketize(const char* aString)
{
   return bracketize(string(aString));
}

