#include "ANamedMutexObject.hpp"
#include "ATrace.hpp"

// Copyright (c) 1997-2000 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@askesis.nl

// Author : Joost Kraaijeveld
// Created: 18 Jan 1998 20:57:58

// public
// Constructors
ANamedMutexObject::ANamedMutexObject(const char* aName)
   :  name(aName)
{
   unsigned long result = 0;

   if (APlatform::isWin32S())
   {
      ATRACE_DEVELOP("Win32s: ANamedMutexObject(const char* aName) does not create semaphore");
      return;
   }

   // If we are running on NT, need to insure we allow sharing to
   // services (i.e. cannot use default security attributes)
   if (APlatform::isWinNT())
   {
      SECURITY_DESCRIPTOR sd;
      SECURITY_ATTRIBUTES sa;

      InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION);
      SetSecurityDescriptorDacl(&sd, TRUE, (PACL) NULL, FALSE);
      sa.nLength = sizeof(sa);
      sa.lpSecurityDescriptor = &sd;
      sa.bInheritHandle = TRUE;

      semaphoreHandle = (AHandle)CreateMutex( &sa, false, (char*)name.c_str() );
   } else
   {
      semaphoreHandle = (AHandle)CreateMutex( 0, false, (char*)name.c_str() );
   }

   if (!semaphoreHandle.isValid())
   {
      result = GetLastError();
      if(result == ERROR_ALREADY_EXISTS)
      {
         semaphoreHandle = (AHandle)OpenMutex(  MUTEX_ALL_ACCESS,
                                                true,
                                                (char*)name.c_str());
         if(!semaphoreHandle.isValid())
         {
            ALibBaseException exception("OpenMutex 1");
            ATHROW(exception);
         }
      }else if (result != 0)
      {
         ALibBaseException exception("OpenMutex 2");
         ATHROW(exception);
      }
   }
}

// Destructor
ANamedMutexObject::~ANamedMutexObject()
{
}

// Operators

// Accessors

// Mutators

// Other operations

// protected
// Operations

// private
// Operations
ANamedMutexObject::ANamedMutexObject(const ANamedMutexObject& aANamedMutexObject)
   :  name(aANamedMutexObject.name)
{
}

ANamedMutexObject& ANamedMutexObject::operator =(const ANamedMutexObject& aANamedMutexObject)
{
   return *this;
}



// End of class definition of ANamedMutexObject
