#ifndef _ARCIPointer_hpp_
#define _ARCIPointer_hpp_

// Copyright (c) 1997-2000 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@askesis.nl

// Author : Joost Kraaijeveld
// Created: 18 Jan 1998 21:35:43
// Purpose: A R(eference)C(ounted)I(ndirect)Pointer class. This class assumes no cooperation from the
//          pointer that it counts.

// Configuration file
#include "ALibBase.h"

// Standard C/C++ library headers

// Operating system toolkit headers

// Open Class Library headers

// Stl headers

// Other class headers

// Forward declarations

template < class T >
class ARCIPointer
{
   public:
      // Constuctors
      ARCIPointer() : referenceCountedPointerHolder(0){}
      ARCIPointer(  T* aRealPointer) : referenceCountedPointerHolder(new ARCIPointerHolder (aRealPointer)){}
      ARCIPointer(  const ARCIPointer < T >&  aARCIPointer)
      {
         referenceCountedPointerHolder = aARCIPointer.referenceCountedPointerHolder;
         if (referenceCountedPointerHolder != 0) referenceCountedPointerHolder->referenceCounter++;
      }
      // Destructor
      virtual ~ARCIPointer()
      {
         if (referenceCountedPointerHolder != 0 && --referenceCountedPointerHolder->referenceCounter == 0)
         {
            delete referenceCountedPointerHolder;
         }
      }
      // Operators
      ARCIPointer < T >& operator=(const ARCIPointer < T >& aReferenceCountedPointer)
      {
         if (this != &aReferenceCountedPointer)
         {
            if (referenceCountedPointerHolder && --referenceCountedPointerHolder->referenceCounter == 0)
            {
               delete referenceCountedPointerHolder;
            }
            referenceCountedPointerHolder = aReferenceCountedPointer.referenceCountedPointerHolder;
            if (referenceCountedPointerHolder != 0)
            {
               referenceCountedPointerHolder->referenceCounter++;
            }
         }
         return *this;
      }
      T* operator->() const
      {
         ASSERT(referenceCountedPointerHolder->realPointer!=0);
         return referenceCountedPointerHolder->realPointer;
      }
      T& operator*() const
      {
         ASSERT(referenceCountedPointerHolder->realPointer!=0);
         return *referenceCountedPointerHolder->realPointer;
      }
      operator T*() const
      {
         return referenceCountedPointerHolder->realPointer;
      }
      bool operator==(  const ARCIPointer < T >& aReferenceCountedPointer) const
      {
         return (*referenceCountedPointerHolder->realPointer)==(*aReferenceCountedPointer.referenceCountedPointerHolder->realPointer);
      }
      bool operator<(const ARCIPointer < T >&  aReferenceCountedPointer) const
      {
         return (*referenceCountedPointerHolder->realPointer)<(*aReferenceCountedPointer.referenceCountedPointerHolder->realPointer);
      }
      bool operator!() const
      {
         if(referenceCountedPointerHolder)
         {
            return false;
         }
         return true;
      }

      // Accessors
      unsigned long                             getReferenceCounter() const
      {
         if(referenceCountedPointerHolder)
         {
            return referenceCountedPointerHolder->referenceCounter;
         }else
         {
            return 0;
         }
      }

      // Mutators

      // Operations

      // Attributes

   protected:
      // Attributes
      struct ARCIPointerHolder
      {
         T* realPointer;
         unsigned long referenceCounter;
         ARCIPointerHolder (T* aRealPointer) :  realPointer (aRealPointer), referenceCounter (1){}
         ~ARCIPointerHolder (){ delete realPointer; }
      } *referenceCountedPointerHolder;

      // Operations
   private:
      // Friends

      // Operations

      // Attributes
}; // End of class declaration of ARCIPointer

#endif
