#ifndef _ARCPointer_hpp_
#define _ARCPointer_hpp_

// Copyright (c) 1997-2000 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@Askesis.nl

// Author : Joost Kraaijeveld
// Created: 18 Jan 1998 21:35:43
// Purpose: AR(eference)C(ounted)Pointer class. This class assumes cooperation from the
//          pointer that it counts by calling addReference() and removeReference() as needed.
//          It redirects the == and < operator to the dereferenced pointers

// Configuration file
#include "ALibBase.h"

// Standard C/C++ library headers

// Operating system toolkit headers

// Stl headers

// Other class headers

// Forward declarations
template < class T >
class ALIBIMPEXP ARCPointer
{
   public:
      // Constructors
      ARCPointer( T* aRealPointer = 0)
         :  realPointer(aRealPointer)
      {}

      ARCPointer( const ARCPointer<T>& source)
         :  realPointer(source.realPointer)
      {
         if(source.realPointer)
         {
            source.realPointer->addReference();
         }
      }

      // Destructor
      virtual ~ARCPointer()
      {
         if(realPointer)
         {
            realPointer->removeReference();
         }
      }

      // Operators
      ARCPointer<T>& operator=(  const ARCPointer<T>& source)
      {
         if ( source.realPointer )
         {
            source.realPointer->addReference();
         }

         if ( realPointer )
         {
            realPointer->removeReference();
         }
         realPointer = source.realPointer;
         return *this;
      }
      ARCPointer<T>& operator=(  T* aRealPointer)
      {
         if(aRealPointer)
         {
            aRealPointer->addReference();
         }
         if(realPointer)
         {
            realPointer->removeReference();
         }
         realPointer = aRealPointer;
         return *this;
      }
      T* operator->()const
      {
         ASSERT(realPointer!=0);
         return realPointer;
      }
      T& operator*() const
      {
         ASSERT(realPointer!=0);
         return *realPointer;
      }
      operator T*() const
      {
         return (realPointer);
      }
      bool operator==(  const ARCPointer<T>& source)
      {
         if(realPointer && source.realPointer)
         {
            // If they point to something than compare the objects
            return (*realPointer) == (*source.realPointer);
         }else
         {
            // else do a pointer comparison as this is alway save (?)
            return realPointer == source.realPointer;
         }
         return false;
      }
      bool operator<(  const ARCPointer<T>& source)
      {
         if(realPointer && source.realPointer)
         {
            // If they point to something than compare the objects
            return (*realPointer) < (*source.realPointer);
         }else
         {
            // else do a pointer comparison as this is alway save (?)
            return realPointer < source.realPointer;
         }
         return false;

      }
      bool operator!() const
      {
         if(realPointer)
         {
            return false;
         }
         return true;
      }

      // Accessors

      // Mutators

      // Other operations

      // Attributes

   protected:
      // Attributes

      // Operations

   private:
      // Friends

      // Operations

      // Attributes
      T*                                        realPointer;
};// End of class declaration of ARCPointer


#endif
