#ifndef _ASynchronizationObject_hpp_
#define _ASynchronizationObject_hpp_

// Copyright (c) 1997-2000 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@Askesis.nl

// Author : Joost Kraaijeveld
// Created: 18 Jan 1998 20:17:44
// Purpose: Base class for all kind of synchronization objects

// Configuration file
#include "ALibBase.h"

// Standard C/C++ library headers

// Operating system toolkit headers

// STL headers

// Other class headers
#include "AHandle.hpp"
#include "APlatform.hpp"

// Forward declarations

class ASynchronizationObjectLock;

class ALIBIMPEXP ASynchronizationObject
{
   public:
      // Constructors
      ASynchronizationObject();

      // Destructor
      virtual ~ASynchronizationObject();

      // Operators

      // Accessors
      AHandle&                                  getHandle();

      // Mutators

      // Operations
      virtual void                              lock( long aTimeOut = -1);
      virtual void                              unlock();

      // Attributes

   protected:
      // Attributes

      // Operations

   private:
      // Friends
      friend class AMutexObject;
      friend class ANamedMutexObject;
      friend class ASemaphoreObject;
      friend class ANamedSemaphoreObject;
      friend class ASynchronizationObjectLock;

      // Operations
      ASynchronizationObject( const ASynchronizationObject& aASynchronizationObject);
      ASynchronizationObject& operator=(const ASynchronizationObject& aASynchronizationObject);

      // Attributes
      AHandle                                   semaphoreHandle;
      unsigned long                             referenceCounter;
      ASynchronizationObjectLock*               synchronizationObjectLock;
}; // End of class declaration of ASynchronizationObject

// Include all the different ASynchronizationObject derived classes and the ASynchronizationObjectLock class
// Clients of this code than only have to include this header.

#include "AMutexObject.hpp"
#include "ANamedMutexObject.hpp"
#include "ASemaphoreObject.hpp"
#include "ANamedSemaphoreObject.hpp"
#include "ASynchronizationObjectLock.hpp"

#endif // _ASynchronizationObject_hpp_