#include "AThread.hpp"
#include "ATrace.hpp"

// Copyright (c) 2000 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@askesis.nl

// Author : Joost Kraaijeveld
// Created:
// Purpose:

#ifdef __ALIBWINDOWS__
/* static */ const AThreadHandle AThreadHandle::nullHandle;    // Whenever a null handle is required
#endif // __ALIBWINDOWS__
/* static */ unsigned long AThread::defaultStackSize = 0;// 32768;  // This value is the same as in the IBM OCL.

// The thread API requires a C-style funtion as the function to start.
// This C-style function is a wrapper around the AThreadFunction run() function.
extern "C" unsigned long WINAPI runMemberFunction(void* aAThreadFunction)
{
   // Development tracing
//   AFUNCTRACE_DEVELOP();

   try
   {
      AThreadFunction* fn = (AThreadFunction*)aAThreadFunction;
      ATRACE_DEVELOP("Before fn->run();");
      fn->run();
      ATRACE_DEVELOP("After fn->run();");
//      ATRACE_DEVELOP("Before ExitThread(0);");
//      ExitThread(0);
//      ATRACE_DEVELOP("After ExitThread(0);");
      return 0;
   }
   catch(AException& exception)
   {
      string text = string("Exception in extern \"C\" void  runMemberFunction(void* aAThreadFunction):\n") + string(exception.text());
      ATRACE_DEVELOP(text.c_str());
      ARETHROW(exception);
   }
   catch(...)
   {
      ATRACE_DEVELOP("Unknown exception in extern \"C\" void  runMemberFunction(void* aAThreadFunction)");
      throw;
   }
   return 0;
}

// Constructors
AThread::AThread()
{
   // Development tracing
   AFUNCTRACE_DEVELOP();
}

AThread::AThread( ACFunctionPtr aACFunctionPtr, void* voidPointerFunctionArgument)
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   start(aACFunctionPtr,
         voidPointerFunctionArgument);
}

AThread::AThread( AThreadFunction* aAFunctionPtr)
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   start(aAFunctionPtr);
}

// Destructor
AThread::~AThread()
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

//   TerminateThread(threadHandle, exitCode);
}

// Operators

// Accessors
/* static */ unsigned long AThread::getDefaultStackSize()
{
   return defaultStackSize;
}

const AThreadId& AThread::getThreadId() const
{
   if(!threadId.isValid())
   {
      string text = string("const AThreadHandle& AThread::getThreadId() const, threadId is not valid");
      ATRACE_DEVELOP(text.c_str());
      ALibBaseException exception(text.c_str());
      ATHROW(exception);
   }

   return threadId;
}

const AThreadHandle& AThread::getThreadHandle() const
{
   if(!threadHandle.isValid())
   {
      string text = string("const AThreadHandle& AThread::getThreadHandle() const, threadHandle is not valid");
      ATRACE_DEVELOP(text.c_str());
      ALibBaseException exception(text.c_str());
      ATHROW(exception);
   }
   return threadHandle;
}

// Mutators
/* static */ void AThread::setDefaultStackSize( unsigned long aDefaultStackSize)
{
   defaultStackSize = aDefaultStackSize;
}

// Other operations
/* static */ AThreadHandle AThread::getCurrentThreadHandle()
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   AThreadHandle aAThreadHandle = GetCurrentThread();
   if(!aAThreadHandle.isValid())
   {
      string text = string("AThread::getCurrentThreadHandle, DuplicateHandle(), GetLastError = ") + integerToString((unsigned long)GetLastError());
      ATRACE_DEVELOP(text.c_str());
      ALibBaseException exception(text.c_str());
      ATHROW(exception);
   }

   return aAThreadHandle;
}

/* static */ ACurrentThread& AThread::current()
{
   static ACurrentThread thread;
   return thread;
}

void AThread::resume()
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   ResumeThread(threadHandle);
}

void AThread::start(ACFunctionPtr aACFunctionPtr, void* voidPointerFunctionArgument)
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   threadHandle = CreateThread(  NULL,                         // pointer to thread security attributes
                                 defaultStackSize,             // initial thread stack size, in bytes
                                 (LPTHREAD_START_ROUTINE)aACFunctionPtr,               // pointer to thread function
                                 voidPointerFunctionArgument,  // argument for new thread
                                 0,                            // creation flags
                                 (DWORD*)&threadId);                   // pointer to returned thread identifier

   if(!threadHandle.isValid())
   {
      string text = string("AThread::start, CreateThread(), GetLastError = ") + integerToString((unsigned long)GetLastError());
      ATRACE_DEVELOP(text.c_str());
      ALibBaseException exception(text.c_str());
      ATHROW(exception);
   }
}

void AThread::start(AThreadFunction* aAFunctionPtr)
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   threadHandle = CreateThread(  NULL,                         // pointer to thread security attributes
                                 defaultStackSize,             // initial thread stack size, in bytes
                                 (LPTHREAD_START_ROUTINE)runMemberFunction,            // pointer to thread function
                                 (void*)aAFunctionPtr,         // argument for new thread
                                 0,                            // creation flags
                                 (DWORD*)&threadId);                   // pointer to returned thread identifier
   if(!threadHandle.isValid())
   {
      string text = string("AThread::start, CreateThread() threadHandle, GetLastError = ") + integerToString((unsigned long)GetLastError());
      ATRACE_DEVELOP(text.c_str());
      ALibBaseException exception(text.c_str());
      ATHROW(exception);
   }
   if(!threadId.isValid())
   {
      string text = string("AThread::start, CreateThread() threadId, GetLastError = ") + integerToString((unsigned long)GetLastError());
      ATRACE_DEVELOP(text.c_str());
      ALibBaseException exception(text.c_str());
      ATHROW(exception);
   }
}

void AThread::stop()
{
   ASSERT(threadHandle.isValid());
   TerminateThread(threadHandle, 0);
}

void AThread::suspend()
{
   ASSERT(threadHandle.isValid());
   SuspendThread(threadHandle);
}

// protected
// Operations

// private
// Operations

// public
// Constructors
ACurrentThread::ACurrentThread()
{
   // Development tracing
   AFUNCTRACE_DEVELOP();
}

// Destructor
ACurrentThread::~ACurrentThread()
{
   // Development tracing
   AFUNCTRACE_DEVELOP();
}

// Operators

// Accessors

// Mutators

// Other operations
void ACurrentThread::suspend()
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   SuspendThread(AThread::getCurrentThreadHandle());
}

void ACurrentThread::sleep(unsigned long aNumberOfMilliSeconds)
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   // The Sleep function suspends the execution of the current thread
   // for a specified interval.
   Sleep(aNumberOfMilliSeconds);
}

void ACurrentThread::waitFor( AThread& aStartedThread)
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   waitFor(aStartedThread,INFINITE);
}

void ACurrentThread::waitFor( AThread& aStartedThread,unsigned long aNumberOfMilliSeconds)
{
   // Development tracing
   AFUNCTRACE_DEVELOP();

   // Don't wait on myself!!
   if(AThread::getCurrentThreadHandle() == aStartedThread.getThreadHandle())
   {
      string text = string("void ACurrentThread::waitFor(...): Cannot wait on myself...");
      ATRACE_DEVELOP(text.c_str());
      ALibBaseException exception(text.c_str());
      ATHROW(exception);
   }
   string text = string("AThread ") + integerToString((unsigned long)GetCurrentThreadId()) + string(" will wait for AThread ") + integerToString(aStartedThread.getThreadId().asUnsigned());
   ATRACE_DEVELOP(text.c_str());

   if(aStartedThread.getThreadHandle().isValid())
   {
      // The WaitForSingleObject function returns when the specified object is in
      // the signaled state or when the time-out interval elapses.

      // Return Value

      // If the function succeeds, the return value indicates the event that caused
      // the function to return.
      // If the function fails, the return value is WAIT_FAILED. To get extended
      // error information, call GetLastError.

      // The return value on success is one of the following values:

      // WAIT_ABANDONED:   The specified object is a mutex object that was not released
      //                   by the thread that owned the mutex object before the owning
      //                   thread terminated. Ownership of the mutex object is granted
      //                   to the calling thread, and the mutex is set to nonsignaled.
      // WAIT_OBJECT_0:    The state of the specified object is signaled.
      // WAIT_TIMEOUT:     The time-out interval elapsed, and the object's state is nonsignaled.

      //Remarks

      // The WaitForSingleObject function checks the current state of the specified object.
      // If the object's state is nonsignaled, the calling thread enters an efficient wait
      // state. The thread consumes very little processor time while waiting for one of the
      // conditions of the wait to be satisfied.
      // The system adjusts the state of some types of objects after the object's signaled
      // state satisfies a wait condition. For example, the count of a semaphore object
      // is decreased by one.
      unsigned long result = WaitForSingleObject(  aStartedThread.getThreadHandle(),   // handle of object to wait for (HANDLE)
                                                   aNumberOfMilliSeconds);             // time-out interval in milliseconds) (DWORD)
      if (result == WAIT_FAILED)
      {
         ALibBaseException exception("ACurrentThread: WaitForSingleObject WAIT_FAILED");
         ATRACE_DEVELOP(exception.text());
         ATHROW(exception);
      }else
      { // Success
         switch(result)
         {
            case WAIT_OBJECT_0:
            {
               // The state of the specified object is signaled.
               // This should be the normal case
               ATRACE_DEVELOP("ACurrentThread: WaitForSingleObject WAIT_OBJECT_0");
               break;
            }
            case WAIT_ABANDONED:
            {
               // The specified object is a mutex object that was not released by the thread that owned the
               // mutex object before the owning thread terminated. Ownership of the mutex object is granted
               // to the calling thread, and the mutex is set to nonsignaled.
               ATRACE_DEVELOP("ACurrentThread: WaitForSingleObject ABANDONED");
               break;
            }
            case WAIT_TIMEOUT:
            {
               // The time-out interval elapsed, and the object's state is nonsignaled.
               ALibBaseException exception("ACurrentThread: WaitForSingleObject WAIT_TIMEOUT");
               ATRACE_DEVELOP(exception.text());
               ATHROW(exception);
               break;
            }
            default:
            {
               ATRACE_DEVELOP("ACurrentThread: WaitForSingleObject UNKNOWN RESULT");
               break;
            }
         }
      }
   }else
   {
      string text = string("void ACurrentThread::waitFor(...): Is the thread started???");
      ATRACE_DEVELOP(text.c_str());
      ALibBaseException exception(text.c_str());
      ATHROW(exception);
   }
}

// protected
// Operations

// private
// Operations
