#ifndef _AThread_hpp_
#define _AThread_hpp_

// Copyright (c) 2000 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@askesis.nl

// Author : Joost Kraaijeveld
// Created:
// Purpose: This file contains a thread class modeled after the IBM OCL thread classes.
//          Most of the interface is compatible so one day (if VA runs on Linux) we may
//          use that class library. Sigh, wish it was on Linux. Best UI class library there is....
//          TODO: 1. Create waitFor(...) functions for ASynchronizationObject's
//                2. What to do if a waitFor is issued before a thread is actualy started?
//
//          NOTICE:  The interface in not the final interface. Keep every use simple.
//                   This is a work in progress.

// Configuration file
#include "ALibBase.h"

// Standard C/C++ library headers
#include <process.h>

// Operating system toolkit headers
#ifdef __ALIBWINDOWS__
#include <windows.h>
#endif // __ALIBWINDOWS__

// STL headers

// Other class headers
#include "AThreadHandle.hpp"
#include "AThreadFunction.hpp"

// Forward declarations
class ACurrentThread;

class ALIBIMPEXP AThread
{
   public:
      // A typedef for a C function that can be run by a thread with default C linkage
      typedef unsigned long WINAPI (* ACFunctionPtr) (void*);

      // Constructors
      AThread();
      AThread( ACFunctionPtr aACFunctionPtr, void* voidPointerFunctionArgument);
      AThread( AThreadFunction* aAFunctionPtr);

      // Destructor
      virtual ~AThread();

      // Operators

      // Accessors
      static unsigned long                      getDefaultStackSize();
      const AThreadId&                          getThreadId() const;
      const AThreadHandle&                      getThreadHandle() const;

      // Mutators
      static void                               setDefaultStackSize( unsigned long aDefaultStackSize);

      // Other operations
      static AThreadHandle                      getCurrentThreadHandle();
      static ACurrentThread&                    current();
      virtual void                              resume();
      void                                      start(ACFunctionPtr aACFunctionPtr, void* voidPointerFunctionArgument);
      void                                      start(AThreadFunction* aAFunctionPtr);
      virtual void                              stop();
      virtual void                              suspend();

      // Attributes

   protected:
      // Attributes

      // Operations

   private:
      // Friends

      // Operations

      // Attributes
      static unsigned long                      defaultStackSize;
      AThreadId                                 threadId;
      AThreadHandle                             threadHandle;
      unsigned long                             exitCode;
}; // End of class declaration of AThread

class ALIBIMPEXP ACurrentThread : public AThread
{
   public:
      // Constructors
      ACurrentThread();

      // Destructor
      virtual ~ACurrentThread();

      // Operators

      // Accessors

      // Mutators

      // Other operations
      virtual void                              suspend();
      virtual void                              sleep(unsigned long aNumberOfMilliSeconds);
      virtual void                              waitFor( AThread& aStartedThread);
      virtual void                              waitFor( AThread& aStartedThread,unsigned long aNumberOfMilliSeconds);

      // Attributes

   protected:
      // Attributes

      // Operations

   private:
      // Friends

      // Operations

      // Attributes
}; // End of class declaration of ACurrentThread


#endif //_AThread_hpp_