#ifndef _ATraceThreadIndention_hpp_
#define _ATraceThreadIndention_hpp_

// Copyright (c) 1997-2000 Askesis B.V. See licence.txt for details.
// For information, bug reports and additions send e-mail to ALib@Askesis.nl

// Author : Joost Kraaijeveld
// Created: 11 Oct 1997 14:22:08
// Purpose: These are some classes that are use to implement tracing per thread.
//
//          Note on the use of operator++(), operator++(int), operator--(), operator++(int)
//
//          Use operator++() (++aAThreadIndentionLevel,prefix) and operator--() (--aAThreadIndentionLevel,prefix)
//          if this patter is followed:
//
//          aAThreadIndentionLevel = aAThreadIndentionLevel+1;
//          doSomethingWithThe(aAThreadIndentionLevel);
//
//          can rewritten as:
//
//          doSomethingWithThe(++aAThreadIndentionLevel);
//
//          Use operator++(int) (aAThreadIndentionLevel++,postfix) and operator--(int) (aAThreadIndentionLevel++,postfix)
//          if this patter is followed:
//
//          doSomethingWithThe(aAThreadIndentionLevel);
//          aAThreadIndentionLevel = aAThreadIndentionLevel+1;
//
//          can rewritten as:
//
//          doSomethingWithThe(aAThreadIndentionLevel++);
//
//          TODO: Should we implement operator+(long anAmount) and operator-(long anAmount)??


// DLL Configuration file
#include "ALibBase.h"

// Standard C/C++ library headers

// Operating system toolkit headers

// STL headers
#include <hash_map>

// Other class headers
#include "AThreadHandle.hpp"

typedef long AIndentionLevel;
typedef long AIndentionLevelAmount;
#define INDENTION_LEVEL 3;

class AThreadIndentionLevel
{
   public:
      AThreadIndentionLevel() : threadId(0),indentionLevel(-3){}
      AThreadIndentionLevel& operator++()    // == ++aAThreadIndentionLevel, prefix
      {
         indentionLevel += INDENTION_LEVEL;
         return *this;
      }
      AThreadIndentionLevel operator++(int)  // == aAThreadIndentionLevel++, postfix
      {
         AThreadIndentionLevel til = *this;
         ++(*this);
         return til;
      }
      AThreadIndentionLevel& operator--()    // == --aAThreadIndentionLevel, prefix
      {
         indentionLevel -= INDENTION_LEVEL;
         return *this;
      }
      AThreadIndentionLevel operator--(int)  // == aAThreadIndentionLevel++, postfix
      {
         AThreadIndentionLevel til = *this;
         --(*this);
         return til;
      }
      AThreadIndentionLevel operator+(AIndentionLevelAmount anAmount)
      {
         AThreadIndentionLevel til(*this);
         til.indentionLevel += anAmount;
         return til;
      }
      AThreadIndentionLevel operator-(AIndentionLevelAmount anAmount)
      {
         AThreadIndentionLevel til(*this);
         til.indentionLevel -= anAmount;
         return til;
      }
      AThreadIndentionLevel& operator+=(AIndentionLevelAmount anAmount)
      {
         indentionLevel += anAmount;
         return *this;
      }
      AThreadIndentionLevel& operator-=(AIndentionLevelAmount anAmount)
      {
         indentionLevel -= anAmount;
         return *this;
      }

      AThreadId         threadId;
      AIndentionLevel   indentionLevel;
};

// Something fishy about std::
//template <class Key> struct hash { };
struct hash< AThreadId >
{
   size_t operator()(const AThreadId aAThreadId) const { return aAThreadId.asUnsigned();}
};

class AThreadIndentionLevels : public hash_map< const AThreadId, AThreadIndentionLevel, hash< AThreadId >, equal_to< AThreadId >  >
{
   public:
      AThreadIndentionLevel& operator[](const AThreadId& aAThreadId)
      {
         if((unsigned long)aAThreadId ==0)
         {
            printf("unsigned long)aAThreadId ==0 ????");
         }

         AThreadIndentionLevel& til = hash_map< const AThreadId, AThreadIndentionLevel, hash< AThreadId >, equal_to< AThreadId > >::operator[](aAThreadId);
         if((unsigned long)til.threadId ==0) // This assumes that there will never be a valid AThreadId 0 !!!!!
         {
            // Insert the new values but first repair the faulty threadId
            til.threadId = aAThreadId;
            hash_map<const AThreadId, AThreadIndentionLevel, hash< AThreadId >, equal_to< AThreadId > >::operator[](aAThreadId) = til;
         }
         return til;
      }
};

#endif // _ATraceThreadIndention_hpp_
