// Configuration file

// Standard C/C++ library headers
#include <iostream>
#include <stdio.h>

// Operating system toolkit headers

// STL headers
#include <string>

// Other class headers
#include "AThreadHandle.hpp"
#include "AThread.hpp"

#define numberOfThreads 2

bool useTrace = false;

class APrintingClass
{
   public:
      APrintingClass(){AFUNCTRACE_DEVELOP();}
      void print0()
      {
         for ( unsigned long i = 0;
               i<5;
               i++)
         {
            printf("%lu\n",i);
            Sleep(100);
         }
      }
      void print1()
      {
         for ( unsigned long i = 0;
               i<5;
               i++)
         {
            string text = integerToString(i);
            ATRACE_DEVELOP(text);
            Sleep(100);
         }
      }
};

extern "C" unsigned long WINAPI runFunction(void* aAThreadFunction)
{
   APrintingClass* p = (APrintingClass*)aAThreadFunction;
   if(useTrace==false)
   {
      p->print0();
   }else
   {
      p->print1();
   }
   return 0;
}


void test1();
void test2();
void test3();

int main(int argc, char* argv[], char* /*envp[]*/)
{
   unsigned long testNumber;

   if(argc==3)
   {
      testNumber  = atoi(argv[1]);
      useTrace    = atoi(argv[2]);
   }else if (argc==2)
   {
      testNumber  = atoi(argv[1]);
      useTrace    = true;
   }else
   {
      printf("Usage: ALibTest testNumber useTrace[=1(true)]");
      printf("Now running as ALibTest 3 1");
      testNumber  = 3;
      useTrace    = true;
   }

   switch(testNumber)
   {
      case 1:
      {
         test1();
         break;
      }
      case 2:
      {
         test2();
         break;
      }
      case 3:
      {
         test3();
         break;
      }
   }
}

void test1()
{
   AThreadHandle threadHandles[numberOfThreads];
   AThreadId threadIds[numberOfThreads];
   APrintingClass objects[numberOfThreads];

   threadHandles[0] = CreateThread( NULL,                   // pointer to thread security attributes
                                    0,                      // initial thread stack size, in bytes
                                    (LPTHREAD_START_ROUTINE)runFunction,            // pointer to thread function
                                    (void*)&objects[0],     // argument for new thread
                                    0,                      // creation flags
                                    (DWORD*)&threadIds[0]);         // pointer to returned thread identifier

   threadHandles[1] = CreateThread( NULL,                   // pointer to thread security attributes
                                    0,                      // initial thread stack size, in bytes
                                    (LPTHREAD_START_ROUTINE)runFunction,            // pointer to thread function
                                    (void*)&objects[1],     // argument for new thread
                                    0,                      // creation flags
                                    (DWORD*)&threadIds[1]);         // pointer to returned thread identifier

   printf("threadHandle: %lu, threadId: %lu\n",threadHandles[0].asUnsigned(),threadIds[0].asUnsigned());
   printf("threadHandle: %lu, threadId: %lu\n",threadHandles[1].asUnsigned(),threadIds[1].asUnsigned());

   printf("Starting to wait....\n");
   unsigned long result = WaitForMultipleObjects(  2,          // number of handles in the object handle array
                                                   (HANDLE*)threadHandles,// pointer to the object-handle array
                                                   true,       // wait flag
                                                   INFINITE);  // time-out interval in milliseconds);
   if (result == WAIT_FAILED)
   {
      printf("WaitForMultipleObjects failed\n");
   }else
   {
      printf("WaitForMultipleObjects succeeded\n");
   }
}

void test2()
{
   AThreadHandle threadHandles[numberOfThreads];
   AThreadId threadIds[numberOfThreads];
   APrintingClass objects[numberOfThreads];

   AThread thread0((AThread::ACFunctionPtr)runFunction,&objects[0]);
   threadHandles[0] = thread0.getThreadHandle();
   threadIds[0] = thread0.getThreadId();

   AThread thread1((AThread::ACFunctionPtr)runFunction,&objects[1]);
   threadHandles[1] = thread1.getThreadHandle();
   threadIds[1] = thread1.getThreadId();

   printf("threadHandle: %lu, threadId: %lu\n",threadHandles[0].asUnsigned(),threadIds[0].asUnsigned());
   printf("threadHandle: %lu, threadId: %lu\n",threadHandles[1].asUnsigned(),threadIds[1].asUnsigned());

   printf("Starting to wait....\n");
   unsigned long result = WaitForMultipleObjects(  2,          // number of handles in the object handle array
                                                   (HANDLE*)threadHandles,// pointer to the object-handle array
                                                   true,       // wait flag
                                                   INFINITE);  // time-out interval in milliseconds);
   if (result == WAIT_FAILED)
   {
      printf("WaitForMultipleObjects failed\n");
   }else
   {
      printf("WaitForMultipleObjects succeeded\n");
   }
}

void test3()
{
   AThreadHandle threadHandles[numberOfThreads];
   AThreadId threadIds[numberOfThreads];
   APrintingClass objects[numberOfThreads];

   AThread thread0(useTrace == false ? new AThreadMemberFunction<APrintingClass>(objects[0],&APrintingClass::print0) : new AThreadMemberFunction<APrintingClass>(objects[0],&APrintingClass::print1));
   threadHandles[0] = thread0.getThreadHandle();
   threadIds[0] = thread0.getThreadId();

   AThread thread1(useTrace == false ? new AThreadMemberFunction<APrintingClass>(objects[1],&APrintingClass::print0) : new AThreadMemberFunction<APrintingClass>(objects[1],&APrintingClass::print1));
   threadHandles[1] = thread1.getThreadHandle();
   threadIds[1] = thread1.getThreadId();

   printf("threadHandle: %lu, threadId: %lu\n",threadHandles[0].asUnsigned(),threadIds[0].asUnsigned());
   printf("threadHandle: %lu, threadId: %lu\n",threadHandles[1].asUnsigned(),threadIds[1].asUnsigned());

   printf("Starting to wait....\n");
   unsigned long result = WaitForMultipleObjects(  2,          // number of handles in the object handle array
                                                   (HANDLE*)threadHandles,// pointer to the object-handle array
                                                   true,       // wait flag
                                                   INFINITE);  // time-out interval in milliseconds);
   if (result == WAIT_FAILED)
   {
      printf("WaitForMultipleObjects failed\n");
   }else
   {
      printf("WaitForMultipleObjects succeeded\n");
   }
}